##### CALCULATE Number of Onset and Offset Neighbours, per Coltheart's N #######
#Usage:
# python coltheartN.py <wordCorpus> <targetList> <cast lowercase: 0/1> <outfile>
#
# Output:
# <outfile> will contain the following information for each word in targetList,
# with each entry placed on a separate line:
#
# <targetWord> <Non> <If neighbours, pipe delimited list of onset neighbours>
#         ...   <Noff> <If neighbours, pipe delimited list of offset neighbours>
#
# This program calculates the number of words in <wordcorpus> that are only
# one letter different from each word in <targetList> either in terms of their
# first (onset) or last (offset) letter.  Unicode support has
# been added so that the program works on non-English characters such as
# French accents.  Accented letters are considered to be different letters.
#
# Requirements: Python 2.7.x
#
# Author: Blair Armstrong, May, 2013

import re
import sys
import codecs

def countNeighbors(word,corpus):
    nonlist=""
    Non = 0

    nofflist = ""
    Noff = 0;

    nlet = len(word)

    #for onset neighbours
    index = 0;
    Non, nonlist = countSpecificPos(index,word,corpus,nlet)

    #for offset neighbours
    index = nlet-1;
    Noff, nofflist = countSpecificPos(index,word,corpus,nlet);

    return Non, nonlist, Noff, nofflist;


def countSpecificPos(index,word, corpus,nlet):
    N = 0;
    nlist = ""

    pat = '^' + word[0:index] + '[\w]' + word[index+1:nlet] + '$'
    m=re.compile(pat,re.UNICODE)

    for entry in corpus:
        if re.match(m,entry) and word != entry:
            N = N + 1
            nlist=nlist+"|"+entry;

    return N, nlist

def main():

    sys.stdout = codecs.getwriter('utf-8')(sys.stdout)

    #Processing Command Line Arguments
    if len(sys.argv) <= 4:
        print 'Usage: python coltheartN.py <wordCorpus> <targetList> \
<cast lowercase: 0/1> <outfile>'
        sys.exit()
    else:
        for arg in sys.argv:
            print arg,
        print ''

    cFile = open(sys.argv[1])
    tFile = open(sys.argv[2])
    toLower = int(sys.argv[3])
    outFile = open(sys.argv[4],"w")

    if toLower != 1 and toLower !=0:
        print 'Error: lowercase cast must be 0/1'
        sys.exit()


    #read the corpus into memory
    corpus = []
    word = cFile.readline()
    while word:
        word = word.strip()

        if toLower == 1:
            word = word.lower()

        corpus.append(word)
        word = cFile.readline()
    cFile.close()


    #lookup the number of neighbors for each target word
    word = tFile.readline()
    while word:
        word = word.strip()

        if toLower == 1:
            word = word.lower()

        Non, nonlist, Noff, nofflist = countNeighbors(word,corpus)
        outFile.write(word + "\t" + str(Non) + "\t" + nonlist + \
                    "\t" + str(Noff) + "\t" + nofflist +  "\n")

        word =tFile.readline()

    outFile.flush();
    outFile.close();
    tFile.close()

main()